#!/bin/sh

doOptional=false

. ./build-env.sh

mkdir -p archives

cd archives

if [ ! -e linux_${lxdebkb}.orig.tar.xz ]; then
  echo "Downloading debian linux_${lxdebkb}..."
  wget -N "${lxdebub}/linux_${lxdebkb}.orig.tar.xz"
  if [ ! -e linux_${lxdebkb}.orig.tar.xz ]; then
    wget "${lxdebub}/linux-${lxdebkb}.tar.xz" -O linux_${lxdebkb}.orig.tar.xz
  fi
  wget -N "${lxdebub}/linux_${lxdebvb}.debian.tar.xz"
  wget -N "${lxdebub}/linux_${lxdebvb}.dsc"
fi

cd ..


if [ ! -e linux-3.14.y-odroidc2 ]; then
  git clone --depth 1 https://github.com/hardkernel/linux.git -b odroidc2-3.14.y linux-3.14.y-odroidc2
fi

cd linux-3.14.y-odroidc2


echo "Packing files from linux-3.14.y-odroidc2..."

tar czf ../archives/linux-3.14-amlogic.tar.gz \
\
arch/arm64/boot/dts/amlogic \
arch/arm64/boot/dts/meson64_odroidc2.dts \
\
arch/arm64/configs/meson64_defconfig \
arch/arm64/configs/odroidc2_defconfig \
\
Documentation/devicetree/bindings/arm/mali-utgard.txt \
Documentation/devicetree/bindings/rng/amlogic,meson.txt \
\
drivers/amlogic \
\
drivers/char/meson-gpiomem.c \
drivers/char/hw_random/meson-rng.c \
\
drivers/clk/clk-scpi.c \
\
drivers/gpu/arm \
drivers/gpu/drm/mali \
\
drivers/hardkernel \
\
drivers/media/rc/meson-ir.c \
drivers/media/v4l2-core/videobuf-res.c \
\
drivers/net/ethernet/stmicro/stmmac/am_eth_reg.h \
drivers/net/ethernet/stmicro/stmmac/dwmac-meson.c \
\
drivers/staging/media/lirc/lirc_odroid.c \
\
drivers/switch \
\
include/dt-bindings/clock/gxbb.h \
include/dt-bindings/clock/meson8.h \
include/dt-bindings/gpio/gxbb.h \
include/dt-bindings/gpio/meson8.h \
include/dt-bindings/reset/aml_gxbb.h \
include/dt-bindings/reset/aml_meson8m2_gate.h \
\
include/linux/amlogic \
include/linux/hardkernel \
\
include/linux/switch.h \
\
include/ump \
\
include/linux/mmc/emmc_partitions.h \
\
include/media/videobuf-res.h \
\
include/uapi/linux/dvb/aml_demod.h \
\
sound/soc/aml


tar czf ../archives/linux-3.14-amlogic-thermal.tar.gz \
\
drivers/cpufreq/cpufreq_hotplug.c \
\
drivers/thermal/cpucore_cooling.c \
drivers/thermal/gpucore_cooling.c \
drivers/thermal/gpu_cooling.c \
\
include/linux/cpucore_cooling.h \
include/linux/gpucore_cooling.h \
include/linux/gpu_cooling.h \
\
drivers/cpufreq/cpufreq_interactive.c \
\
drivers/thermal/devfreq_cooling.c \
drivers/thermal/power_allocator.c \
\
include/linux/devfreq_cooling.h \
\
include/trace/events/cpufreq_interactive.h \
\
include/uapi/linux/thermal.h


tar czf ../archives/linux-3.14-amlogic-part2-for-3.16.tar.gz \
\
arch/arm64/include/asm/opcodes.h \
arch/arm64/include/asm/scatterlist.h \
\
arch/arm64/kernel/image.h \
arch/arm64/kernel/opcodes.c \
arch/arm64/kernel/swp_emulate.c \
\
Documentation/devicetree/bindings/mailbox \
Documentation/devicetree/changesets.txt \
Documentation/devicetree/dynamic-resolution-notes.txt \
Documentation/mailbox.txt \
Documentation/sync.txt \
\
drivers/mailbox/mailbox.c \
\
drivers/staging/keucr/init.h \
\
drivers/usb/host/ehci-h20ahb.c \
\
include/linux/cma.h \
include/linux/mailbox_client.h \
include/linux/mailbox_controller.h \
include/linux/pl320-ipc.h \
\
include/linux/usb/xhci_pdriver.h \
\
mm/cma.c \
\
sound/soc/codecs/pcm5102.c


cd ..

echo "Extracting files from linux ${lxdebkb}..."

tar xJf archives/linux_${lxdebkb}.orig.tar.xz && \
mv linux-${lxdebkb} linux && \
cd linux && \
tar xJf ../archives/linux_${lxdebvb}.debian.tar.xz && \
mv debian/patches .

echo "Patching linux-${lxdebvb}..."
p=linux-${DEB_ABINAME}
quilt push -a 2>&1 1>../${p}.patch.log
r=$?

if [ $r != 0 ]; then
  cat ../${p}.patch.log | grep 'rejects in file' | cut -d - -f 3- | cut -d " " -f 5
  echo Failed $r
  exit $r
fi

sed -i "s/abiname: $DEB_ABI_STEP/abiname: $BSP_ABI_STEP/g" debian/config/defines

sed -i 's|## Run user hook script here, if any|system ("cp -p /usr/lib/$package_name/*.dtb /media/boot/");|g'  debian/templates/image.plain.postinst.in

echo "Extracting files from linux-3.14.y-odroidc2..."

if [ -e drivers/thermal/thermal_core.h -a ! -e include/linux/thermal_core.h ]; then
  mv drivers/thermal/thermal_core.h  include/linux/
  cat include/linux/thermal_core.h | grep -m1 -B25 '\*/$' > drivers/thermal/thermal_core.h
  echo "" >> drivers/thermal/thermal_core.h
  echo "#include <linux/thermal_core.h>" >> drivers/thermal/thermal_core.h
fi

if [ -e include/dt-bindings -a ! -e arch/${BSP_ARCH}/boot/dts/include/dt-bindings ]; then
  mkdir -p arch/${BSP_ARCH}/boot/dts/include
  ln -s ../../../../../include/dt-bindings arch/${BSP_ARCH}/boot/dts/include/dt-bindings
fi

tar xzf ../archives/linux-3.14-amlogic.tar.gz
tar xzf ../archives/linux-3.14-amlogic-part2-for-3.16.tar.gz

mkdir patches/platform

cp -apP ../patches/. patches/platform/
cat patches/platform/series-odroidc2 >> patches/series
quilt push -a

if $doOptional ; then
  tar xzf ../archives/linux-3.14-amlogic-thermal.tar.gz

  cp -apP ../patches-optional/. patches/platform/
  cat patches/platform/series-odroidc2-optional >> patches/series
  quilt push -a
fi

cat patches/platform/config-odroidc2 >> debian/config/${BSP_ARCH}/config

mkdir -p debian/config/kernelarch-${BSP_ARCH}
cp -p debian/config/${BSP_ARCH}/config debian/config/kernelarch-${BSP_ARCH}/config.${BSP_MODEL}
cat patches/platform/config-odroidc2 >> debian/config/kernelarch-${BSP_ARCH}/config.${BSP_MODEL}


cat <<EOFDEF | tee debian/config/${BSP_ARCH}/defines > /dev/null
[base]
flavours:
 ${DEB_MODEL}
 ${BSP_MODEL}
kernel-arch: ${BSP_ARCH}

[build]
debug-info: true
image-file: arch/${BSP_ARCH}/boot/Image

[image]
install-stem: vmlinuz

[${DEB_MODEL}_description]
hardware: 64-bit ARMv8 machines

[${DEB_MODEL}_image]

[${BSP_MODEL}_description]
hardware: C2 machines
hardware-long: Odroid C2 machines

[${BSP_MODEL}_image]
configs: kernelarch-${BSP_ARCH}/config.${BSP_MODEL}
EOFDEF

#export ARCH=arm64
#export LOCALVERSION="+c2"

#make LOCALVERSION="${LOCALVERSION}" ARCH=${ARCH} odroidc2_defconfig

echo OK
